;*******************************************************************************
;*
;*      Titel:  Unterprogramm zur Nutzung des internen EEPROM
;*              (SR_EEPROM.asm)
;*
;*      Datum:  13/02/12        Version:        0.2 (12/12/13)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A/32u4/644p     Quarzfrequenz:  16MHz
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*      EEPRWR: Unterprogramm zum Schreiben eines Datenbyte
;*              Die Adresse muss sich im Doppelregister 25:r24 befinden
;*              Das Datenbyte im SRAM        (log. Adresse: EEPDAT)
;*      EEPRRD: Unterprogramm zum Lesen eines Datenbyte
;*              Die Adresse muss sich im Doppelregister 25:r24 befinden
;*              Das Datenbyte landet im SRAM (log. Adresse: EEPDAT)
;*
;*******************************************************************************

;///////////////////////////////////////////////////////////////////////////////
;	Makros 
;	siehe http://www.mikrocontroller.net/articles/AVR_Assembler_Makros
;///////////////////////////////////////////////////////////////////////////////
;Bei neuen Chips liegen viele SF-Register ueber den 64 Adressen die mit den 
;Befehlen "in" und "out" zu erreichen sind. Fuer sie muessen "sts" und "lds"
;verwendet werden. Dies wird mit den folgenden beiden Makros erledigt. Statt der
;Befehle "in" und "out" werden in der Bibliothek die Makros "input" und "output"
;verwendet.

;Einlesen eines Ports mit input Arbeitsregister,SF-Register
.IFNDEF MACROINPUT
.SET	MACROINPUT = 1
  .MACRO input
    .IF @1 < 64
	in	@0, @1
    .ELSE
	lds	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;Schreiben eines Ports mit output SF-Register,Arbeitsregister
.IFNDEF MACROOUTPUT
.SET	MACROOUTPUT = 1
  .MACRO output
    .IF @0 < 64
	out	@0, @1
    .ELSE
  	sts	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
EEPDAT:	 .BYTE 1		;wird zur Datenuebergabe (schreiben) benoetigt	
.CSEG

;-------------------------------------------------------------------------------
;       Datenbyte ins interne EEPROM schreiben
;       Adresse in r25:r24
;       Datenbyte im SRAM (EEPDAT)
;-------------------------------------------------------------------------------
EEPRWR: push	r24
	push	r25	
.IFDEF	EEPM0			;mega32u4, mega 644p
	cbi	EECR,4		;Erase and Write in one operation 
	cbi	EECR,5		;(Atomic Operation) EEPM0 = EEPM1 = 0	
.ENDIF
EEPRWL:	sbic	EECR,1		;Polling EEWE (EEPROM Write Enable) bzw. EEPE	
	rjmp	EEPRWL
	cli			;sicherheitshalber Interrupts sperren
	output	EEARH,r25	;Adresse schreiben
	output	EEARL,r24
	lds	r24,EEPDAT	
	output	EEDR,r24	;Datenbyte ins Datenregister
	sbi	EECR,2		;starte Schreibzugriff (EEMWE bzw. EEMPE = 1)
	sbi	EECR,1		;starte Schreibvorgang (EEWE bzw. EEPE = 1)
	sei			;Interrupts wieder freigeben
	pop	r25
	pop	r24
        ret

;-------------------------------------------------------------------------------
;       Datenbyte aus dem interne EEPROM lesen
;       Adresse in r25:r24
;       Datenbyte im SRAM (EEPDAT)
;-------------------------------------------------------------------------------
EEPRRD:	push	r24
	push	r25	
EEPRRL:	sbic EECR,1		;Polling EEWE (EEPROM Write Enable) bzw. EEPE	
	rjmp EEPRRL
	cli			;sicherheitshalber Interrupts sperren
	output	EEARH,r25	;Adresse schreiben
	output	EEARL,r24	
	sbi EECR,0		;starte Lesezugriff (EERE = 1)
	input	r24,EEDR	;Datenbyte ins Datenregister
	sts	EEPDAT,r24
	sei			;Interrupts wieder freigeben
	pop	r25
	pop	r24
	ret        
