;*******************************************************************************
;*
;*      Titel:  Unterprogramme zur Nutzung der seriellen Schnittstelle
;*              (SR_EIA232.asm)
;*
;*      Datum:  04/02/12        Version:        0.2
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A/644p/32u4	Quarzfrequenz:  
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*      
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;Damit der Code im Flash gering bleibt werden ausschliesslich die benutzten
;Unterprogramme freigeschaltet (Flag = 1)
.EQU    Flag_E232INI  = 1	;Schnittstelle initialisieren (default = 1)
.EQU    Flag_E232SND  = 1	;Polling Senden
.EQU    Flag_E232RCV  = 1	;Polling Empfang
.EQU    Flag_E232HEX  = 1	
.EQU    Flag_ISRTX    = 1	;Interrupt Senden
.EQU    Flag_ISRRX    = 1	;Interrupt Empfang
.EQU    Flag_E232CL   = 1	;"command list" ausgeben

;///////////////////////////////////////////////////////////////////////////////
;	Makros 
;	siehe http://www.mikrocontroller.net/articles/AVR_Assembler_Makros
;///////////////////////////////////////////////////////////////////////////////
;Bei neuen Chips liegen viele SF-Register ueber den 64 Adressen die mit den 
;Befehlen "in" und "out" zu erreichen sind. Fuer sie muessen "sts" und "lds"
;verwendet werden. Dies wird mit den folgenden beiden Makros erledigt. Statt der
;Befehle "in" und "out" werden in der Bibliothek die Makros "input" und "output"
;verwendet.

;Einlesen eines Ports mit input Arbeitsregister,SF-Register
.IFNDEF MACROINPUT
.SET	MACROINPUT = 1
  .MACRO input
    .IF @1 < 64
	in	@0, @1
    .ELSE
	lds	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;Schreiben eines Ports mit output SF-Register,Arbeitsregister
.IFNDEF MACROOUTPUT
.SET	MACROOUTPUT = 1
  .MACRO output
    .IF @0 < 64
	out	@0, @1
    .ELSE
  	sts	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
RXFLAG:	.BYTE 1			;wird von der Routine ISRRX benoetigt	
.CSEG

; Achtung Datenblatt beachten!
; Nicht alle Kombinationen liefern eine fehlerfreie Uebertragung
; Bei 16 MHz sind 9600, 19200 und 38400 Bit/s gute Werte
.EQU    EIA_Baud = 9600         ;Bitrate
.IFNDEF CPU_Clock
.EQU    CPU_Clock = 16000000    ;Systemtakt
.ENDIF

;ATmega32U4  !!! Achtung 1 serielle Schnittstelle USART1 an PD2 und PD3 !!!
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x95) && (SIGNATURE_002 == 0x87))
.EQU    UCSRA   = UCSR1A
.EQU    UCSRB   = UCSR1B
.EQU    UCSRC   = UCSR1C
.EQU    UBRRL   = UBRR1L
.EQU    UBRRH   = UBRR1H
.EQU    UDR     = UDR1
.EQU    UDRE    = UDRE1
.EQU    TXEN    = TXEN1
.EQU    RXEN    = RXEN1
.EQU    UDRIE   = UDRIE1
.EQU    RXCIE  =  RXCIE1
.ENDIF

;ATmega644P  !!! 2 serielle Schnittstelle USART0 an PD0 und PD1 (USART1 PD2+3)!!
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x96) && (SIGNATURE_002 == 0x0A))
.EQU    UCSRA   = UCSR0A
.EQU    UCSRB   = UCSR0B
.EQU    UCSRC   = UCSR0C
.EQU    UBRRL   = UBRR0L
.EQU    UBRRH   = UBRR0H
.EQU    UDR     = UDR0
.EQU    UDRE    = UDRE0
.EQU    TXEN    = TXEN0
.EQU    RXEN    = RXEN0
.EQU    UDRIE   = UDRIE0
.EQU    RXCIE   = RXCIE0
.ENDIF

;-------------------------------------------------------------------------------
;       EIA 232 (USART) initialisieren (Datenformat 8N1)
;-------------------------------------------------------------------------------
.IF Flag_E232INI == 1
E232INI: push	r16
	;Sender und Empfaenger einschalten und Datenformat festlegen
.IF ((Flag_ISRTX == 1) || (Flag_E232SND  == 1))
	input   r16,UCSRB
        ori	r16,0x08        ;Sender einschalten (TxEnable, Bit 3)
        output  UCSRB,r16        	
.ENDIF
.IF ((Flag_ISRRX == 1) || (Flag_E232RCV  == 1))
        input   r16,UCSRB
        ori	r16,0x10	;Empfaenger einschalten (RxEnable Bit 4)
        output  UCSRB,r16        
.ENDIF
.IF Flag_ISRRX == 1                
        input   r16,UCSRB
        ori	r16,0x80	;Empfangsinterrupt (RXCIE) freigeben
        output  UCSRB,r16        			
.ENDIF
	output  UCSRB,r16        			
        ldi     r16,0x86        ;UCSRC = 10000110b
        output  UCSRC,r16       ;Waehle Asynchr. 8N1
                                ;B7 URSEL=1 UCSRC ausgewaehlt
                                ;B6 UMSEL=0 Asynchron
                                ;B45 UPM=00 keine Paritaet
                                ;B3 USBS=0 1 Stoppbit
                                ;B21 UCSZ=11 8 Datenbit
                                ;B0 UCPOL=0 bei Asyn.
        ;Baudrate initialisieren (Teiler fuer Baudrate festlegen)
        ldi     r16,HIGH((CPU_Clock/(EIA_Baud*16))-1)        
        output  UBRRH,r16              
        ldi     r16,LOW((CPU_Clock/(EIA_Baud*16))-1)
        output  UBRRL,r16  
        
.IF Flag_ISRRX == 1        
	sts	RXFLAG,Zero	;Flag fuer die Routine ISRRX loeschen	
.ENDIF                
        pop	r16
        ret
.ENDIF        
;===============================================================================
;       Polling
;===============================================================================
;-------------------------------------------------------------------------------
;       Senden eines Zeichens (Parameter r24)
;-------------------------------------------------------------------------------
.IF Flag_E232SND == 1
E232SND:
	push	r16 
E232SL: input	r16,UCSRA	;Warte bis UDREmpty gesetzt ist, da der Schreib-
	sbrs	r16,UDRE	;versuch sonst ignoriert wird        	
        rjmp    E232SL         
        output  UDR,r24         ;und gib das Zeichen aus        
        pop	r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Senden eines Bytes in Hexadezimal (Parameter r24)
;       Ausgabe: 2 Zeichen+Space)
;-------------------------------------------------------------------------------        
.IF Flag_E232HEX == 1
E232HEX:
	push	r16	
	push	r17
	push	r24
	push	r25
	
	ldi	r16,0x30	;ASCII Offset Ziffer
	ldi	r17,0x37	;ASCII Offset Grossbuchstabe
	
	mov	r25,r24
	andi	r24,0xF0
	swap	r24	
	cpi	r24,10
	brge	E232H1
	add	r24,r16	
	rjmp	E232H2
E232H1:	add	r24,r17	
E232H2:	rcall	E232SND	
	mov	r24,r25
	andi	r24,0x0F
	cpi	r24,10
	brge	E232H3
	add	r24,r16	
	rjmp	E232H4
E232H3:	add	r24,r17		
E232H4:	rcall	E232SND	
	ldi	r24,' '
	rcall	E232SND
	
	pop	r25
	pop	r24
	pop	r17
	pop	r16
	ret
.ENDIF        

;===============================================================================
;       Interrupt
;===============================================================================
;-------------------------------------------------------------------------------
;	Interrupt Routine zum Senden eines Zeichens ueber EIA232
;	
;	Im Hauptprogramm muss der Interruptvektor initialisiert werden:
;	.ORG	UDREaddr (bzw. UDRE1addr)
;		rjmp	ISRTX
;	
;	E232INI muss in der Initialisierungsphase aufgerufen werden
;		rcall	E232INI
;		
;	Das spezifische Interrupt muss kurz vor dem Senden freigeschaltet
;	werden. Der Text befindet sich in einer Flash-Tabelle und schliesst
;	mit dem Nullbyte ab. Die Tabelle muss ensprechend initialisiert werden.
;	
;		ldi	ZL,LOW(TEXT*2)	;Tabellenzeiger initialisieren
;		ldi	ZH,HIGH(TEXT*2)
;		input   r16,UCSRB	;Interrupt (UDRIE) freischalten
;       	ori	r16,0x20         
;       	output  UCSRB,r16        	
;	TEXT:	.DB	"Hallo",0
;-------------------------------------------------------------------------------
.IF Flag_ISRTX == 1
ISRTX:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;

	lpm	r16,Z+		;Zeichen aus der Tabelle einlesen
	cpi	r16,0		;Falls Nullbyte, dann raus
	breq	ISRTXC			
	output	UDR,r16		;Zeichen ausgeben
	rjmp	ISRTXR
	
ISRTXC:	input   r16,UCSRB	;Interrupt sperren (UDRIE loeschen)
        andi	r16,0xDF         
       	output  UCSRB,r16        	

ISRTXR:	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti    		;zurueck und Interrupts wieder freigeben
.ENDIF    

;-------------------------------------------------------------------------------
;	Interrupt Routine zum Empfang von Zeichen (Befehlen) ueber EIA232
;	
;	;	Im Hauptprogramm muss der Interruptvektor initialisiert werden:
;	.ORG	URXCaddr (bzw.URXC1addr)
;		rjmp	ISRRX
;	
;	E232INI muss in der Initialisierungsphase aufgerufen werden
;		rcall	E232INI
;	
;	Das obere Nibble des Zeichenzaehlers (RXCFLAG im SRAM) definiert den
;	Befehl (1-15), das untere Nibble das momentan empfangene Zeichen des
;	Befehls. Befehle muessen mit unterschiedlichen Buchstaben beginnen!
;	Ist der Befehl vollstaendig empfangen worden, wird 0xF ins untere Nibble
;	geschrieben. Dies dient als Flag im Hauptprogramm um den Befehl zu
;	verarbeiten.
;	Die Routine muss natuerlich entsprechend den Anforderungen veraendert
;	werden
;	
;	Beispielroutine:
;	1 Befehl: "?" fuer help
;	2 Befehl: "g" fuer "get data" Die aktuellen Daten sollen gesendet werden          
;	3 Befehl: "q" fuer "get quarter data" Die aktuellen Daten
;	           (Viertelstunde) sollen gesendet werden          
;	4 Befehl: "d" fuer "get day data" Die aktuellen Daten
;	           (Tageswerte) sollen gesendet werden         
;	5 Befehl: "aq" fuer "all data quarter" Alle Daten des EEPROMS Viertel-
;		  stundenwerte sollen zurueckgesendet werden         
;	6 Befehl: "ad" fuer "all data day" Alle Daten des EEPROMS Tages-
;		  werte sollen zurueckgesendet werden         
;	7 Befehl: "p" beide EEPROM-Zeiger ausgeben (Q fuer Viertelstundenbereich 
;		  auf: 0x0000:0x0001; D fuer Tagesbereich auf 0x0002:0x0003)
;	8 Befehl: "t 2204121522" fuer "set time" speichert das Datum (dd.mm.jj)
;		   und die Uhrzeit gleich im RTC ab. Hier wird das Flag sofort
;		   geloescht.
;	9 Befehl: "s 0000E000 set pointer setzt die beider Adresszeiger
;	          (interner EEPROM 0x0000:0x0001 0x0002:0x0003
;	
;	          
;-------------------------------------------------------------------------------
.IF Flag_ISRRX == 1
ISRRX:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;
	push    r17    
	push    r18    
		
	input   r16,UDR         ;Zeichen einlesen
;	output	UDR,r16		;echo zum Testen

	lds	r17,RXFLAG
	cpi	r17,0x00
	brne	ISRRX0
	cpi	r16,'?'		;Sortiere die Befehle
	brne	ISRRXA		
	rjmp	ISRRX1
ISRRXA:	cpi	r16,'g'		
	brne	ISRRXB
	rjmp	ISRRX2
ISRRXB:	cpi	r16,'q'	
	brne	ISRRXC
	rjmp	ISRRX3
ISRRXC:	cpi	r16,'d'	
	brne	ISRRXD
	rjmp	ISRRX4
ISRRXD:	cpi	r16,'a'	
	brne	ISRRXE
	rjmp    ISRRX5 		;5 und 6 Befehl! 
ISRRXE:	cpi	r16,'p'	
	brne	ISRRXF
	rjmp    ISRRX7  
ISRRXF:	cpi	r16,'t'	
	brne	ISRRXG
	rjmp    ISRRX8  
ISRRXG:	cpi	r16,'s'	
	brne	ISRRXH
	rjmp    ISRRX9  
ISRRXH: cpi     r16,'c' 
        brne    ISRRX0R         ;raus
        rjmp    ISRRX10  

ISRRX0: andi	r17,0xF0	;falls ein Befehl mehr als 1 Buchstabe	
	cpi	r17,0x50	;enthaelt (hier der 5 (6), 8, und 9 Befehl)!
	brne 	ISRRX01
	rjmp	ISRRX51
ISRRX01: cpi	r17,0x80	
	brne 	ISRRX02
	rjmp	ISRRX81
ISRRX02: cpi	r17,0x90	
	brne 	ISRRX0R
	rjmp	ISRRX91
ISRRX0R: rjmp	ISRRXR		;raus

        ;1 Befehl
ISRRX1:	ldi	r17,0x1F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;2 Befehl
ISRRX2:	ldi	r17,0x2F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;3 Befehl
ISRRX3:	ldi	r17,0x3F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus
	
	;4 Befehl
ISRRX4:	ldi	r17,0x4F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;5 und 6 Befehl
ISRRX5:	ldi	r17,0x51	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus
ISRRX51: cpi	r16,'q'	
	brne	ISRRX52
	ldi	r17,0x5F	
	sts	RXFLAG,r17	
ISRRX52: cpi	r16,'d'	
	brne	ISRRX53
	ldi	r17,0x6F	
	sts	RXFLAG,r17	
ISRRX53: rjmp	ISRRXR		;raus

	;7 Befehl
ISRRX7:	ldi	r17,0x7F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;8 Befehl
ISRRX8:	ldi	r17,0x81
	sts	RXFLAG,r17
	rjmp	ISRRXR
ISRRX81: cpi	r16,' '	
	brne	ISRRX82
	lds	r17,RXFLAG
	cpi	r17,0x81
	brne	ISRRX82	
	inc	r17	
	sts	RXFLAG,r17
	rjmp	ISRRXR
	
	;Tag
ISRRX82: lds	r17,RXFLAG
	cpi	r17,0x82
	brne	ISRRX83
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+4,r16
	rjmp	ISRRXR

ISRRX83: lds	r17,RXFLAG
	cpi	r17,0x83
	brne	ISRRX84
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+4
	add	r16,r17
        sts     RTCTAB+4,r16
        rjmp	ISRRXR

	;Monat
ISRRX84: lds	r17,RXFLAG
	cpi	r17,0x84
	brne	ISRRX85
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+5,r16
	rjmp	ISRRXR

ISRRX85: lds	r17,RXFLAG
	cpi	r17,0x85
	brne	ISRRX86
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+5
	add	r16,r17
        sts     RTCTAB+5,r16
        rjmp	ISRRXR

        ;Jahr
ISRRX86: lds	r17,RXFLAG
	cpi	r17,0x86
	brne	ISRRX87
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+6,r16
 	rjmp	ISRRXR

ISRRX87: lds	r17,RXFLAG
	cpi	r17,0x87
	brne	ISRRX88
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+6
	add	r16,r17
        sts     RTCTAB+6,r16
        rjmp	ISRRXR

        ;Stunden
ISRRX88: lds	r17,RXFLAG
	cpi	r17,0x88
	brne	ISRRX89
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+2,r16
	rjmp	ISRRXR

ISRRX89: lds	r17,RXFLAG
	cpi	r17,0x89
	brne	ISRRX8A
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+2
	add	r16,r17
        sts     RTCTAB+2,r16
        rjmp	ISRRXR
        
        ;Minuten
ISRRX8A: lds	r17,RXFLAG
	cpi	r17,0x8A
	brne	ISRRX8B
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+1,r16
 	rjmp	ISRRXR

ISRRX8B: lds	r17,RXFLAG
	cpi	r17,0x8B
	breq	ISRRX8C	
	rjmp	ISRRXR	
ISRRX8C: subi	r16,0x30
	lds	r17,RTCTAB+1
	add	r16,r17
        sts     RTCTAB+1,r16        
        ;Sekunden und Control Byte         
        sts     RTCTAB+0,Zero
        sts     RTCTAB+7,Zero	
        ldi	r17,0x8F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus
;        ;RTC sofort setzen um Verzoegerungen zu vermeiden
;        rcall   RTCSND		;RTC setzen        
;       sts	RXFLAG,Zero	;RXFLAG zuruecksetzen
;        rjmp	ISRRXR

	;9 Befehl
ISRRX9:	ldi	r17,0x91
	sts	RXFLAG,r17
	rjmp	ISRRXR
ISRRX91: cpi	r16,' '	
	brne	ISRRX92
	lds	r17,RXFLAG
	cpi	r17,0x91
	brne	ISRRX92	
	inc	r17	
	sts	RXFLAG,r17
	rjmp	ISRRXR
	
ISRRX92: ;1 Byte
	lds	r17,RXFLAG
	cpi	r17,0x92
	brne	ISRRX93
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+0,r16
	rjmp	ISRRXR  

ISRRX93: lds	r17,RXFLAG
	cpi	r17,0x93
	brne	ISRRX94
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+1,r16
        rjmp	ISRRXR

	;2 Byte
ISRRX94: lds	r17,RXFLAG
	cpi	r17,0x94
	brne	ISRRX95
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+2,r16
	rjmp	ISRRXR

ISRRX95: lds	r17,RXFLAG
	cpi	r17,0x95
	brne	ISRRX96
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+3,r16
        rjmp	ISRRXR

        ;3 Byte
ISRRX96: lds	r17,RXFLAG
	cpi	r17,0x96
	brne	ISRRX97
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+4,r16
	rjmp	ISRRXR

ISRRX97: lds	r17,RXFLAG
	cpi	r17,0x97
	brne	ISRRX98
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+5,r16
        rjmp	ISRRXR

        ;4 Byte
ISRRX98: lds	r17,RXFLAG
	cpi	r17,0x98
	brne	ISRRX99
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	sts	EEPPOI+6,r16
	rjmp	ISRRXR

ISRRX99: lds	r17,RXFLAG
	cpi	r17,0x99
	brne	ISRRXR
	sts	EEPPOI+7,r16
	ldi	r17,0x9F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

        ;10 Befehl	
ISRRX10: ldi     r17,0xAF        
        sts     RXFLAG,r17      
        rjmp    ISRRXR          ;raus

ISRRXR:	pop	r18
	pop	r17
	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti    		;zurueck und Interrupts wieder freigeben
.ENDIF    


;-------------------------------------------------------------------------------
;	"command list" ausgeben          
;-------------------------------------------------------------------------------
.IF Flag_E232CL == 1
E232CL:	push	r16

	ldi	ZL,LOW(TEXT1*2)	;Tabellenzeiger initialisieren
	ldi	ZH,HIGH(TEXT1*2)
	input   r16,UCSRB	;Interrupt (UDRIE) freischalten
      	ori	r16,0x20         
      	output  UCSRB,r16    

      	pop	r16
	ret


TEXT1:	.DB	"command list:",0x0D
	.DB	"?  help",0x0D
	.DB	"g  get data (1 set)",0x0D
	.DB	"q  get quarter data (last page)",0x0D
	.DB	"d  get day data (last page)",0x0D
	.DB	"aq get all quarter data (448 pages)",0x0D
	.DB	"ad get all day data (64 pages) ",0x0D
	.DB	"p  get the 2 EEPROM Pointer (quarter + day)",0x0D
	.DB	"t  set RTC ",0x0D
	.DB	"   format:  t+space+date(ddmmjj)+time(hhmm)",0x0D
	.DB	"   example: t 2204121522 ",0x0D
	.DB	"s  set the 2 EEPROM Pointer",0x0D
	.DB	"   format:  s+space+2byte(1pointer)+2byte(2pointer)",0x0D
	.DB	"   example: s 0000E000 ",0x0D
	.DB     "c  clear all and reset pointer",0x0D,0

.ENDIF    