;*******************************************************************************
;*
;*      Titel:  Unterprogrammbibliothek zur Ganzzahlmultiplikation, Division
;*              und zum Umcodieren
;*              (SR_MATH.asm)
;*
;*      Datum:  15/05/10        Version:        0.6 (14/01/14)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A/644p/32u4	Quarzfrequenz:  
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*      
;*      DIV21   Ganzahldivision 2Byte/1Byte   
;*      DIV32   Ganzahldivision 3Byte/2Byte   
;*      DIV42   Ganzahldivision 4Byte/2Byte
;*      DIV62   Ganzahldivision 6Byte/2Byte
;*      MUL22   Ganzahlmultiplikation 2Byte*2Byte   
;*      MUL42   Ganzahlmultiplikation 4Byte*2Byte   
;*      MUL22S  Ganzzahlmultiplikation signed 2Byte*2Byte   
;*      MUL42S  Ganzzahlmultiplikation signed 4Byte*2Byte   
;*      
;*      HEX2A2  HEX nach ASCII 2 Byte
;*      BCD2A2  BCD nach ASCII 2 Byte
;*      DEC2A2  Dezimal bis 99 nach ASCII 2 Byte
;*              (benoetigt DIV21)
;*      DEC2A3  Dezimal bis 255 nach ASCII 3 Byte + Nullbyte
;*              (benoetigt DIV21 und DIV32)
;*      DEC2A5  Dezimal bis 65536 nach ASCII 5 Byte + Nullbyte
;*              (benoetigt DIV21 und DIV32)
;*      ASCB2D2 ASCII (BCD,2 Byte) nach Dezimal/Binaer
;*      ASCH2D2 ASCII (HEX,2 Byte) nach Dezimal/Binaer
;*      ASC2BD4 ASCII (BCD,4 Byte) nach Dezimal/Binaer
;*      
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
;Damit der Code im Flash gering bleibt werden ausschliesslich die benutzten
;Unterprogramme freigeschaltet (Flag = 1)
.EQU    Flag_DIV21  = 1         ;Ganzahldivision 2Byte/1Byte   
.EQU    Flag_DIV32  = 1         ;Ganzahldivision 3Byte/2Byte   
.EQU    Flag_DIV42  = 0         ;Ganzahldivision 4Byte/2Byte   
.EQU    Flag_DIV62  = 0         ;Ganzahldivision 6Byte/2Byte   
.EQU    Flag_MUL22  = 0         ;Ganzahlmultiplikation 2Byte*2Byte   
.EQU    Flag_MUL42  = 0         ;Ganzahlmultiplikation 4Byte*2Byte   
.EQU    Flag_MUL22S = 0         ;Ganzzahlmultiplikation signed 2Byte*2Byte   
.EQU    Flag_MUL42S = 0         ;Ganzzahlmultiplikation signed 4Byte*2Byte   

.EQU    Flag_HEX2A2 = 1		;HEX nach ASCII 2 Byte
.EQU    Flag_BCD2A2 = 1		;BCD nach ASCII 2 Byte
.EQU    Flag_DEC2A2 = 0		;Dezimal bis 99 nach ASCII 2 Byte
.EQU    Flag_DEC2A3 = 1         ;Dezimal bis 255 nach ASCII 3 Byte + Nullbyte
.EQU    Flag_DEC2A5 = 1		;Dezimal bis 65536 nach ASCII 5 Byte + Nullbyte
.EQU    Flag_ASCB2D2 = 1	;ASCII (BCD,2 Byte) nach Dezimal/Binaer
.EQU    Flag_ASCH2D2 = 1	;ASCII (HEX,2 Byte) nach Dezimal/Binaer
.EQU    Flag_ASCB2D4 = 0        ;ASCII (BCD,4 Byte) nach Dezimal/Binaer
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
TDEC2A:	.BYTE	6		;Tabelle fuer ASCII-Zeichen DEC2A5
.CSEG
;-------------------------------------------------------------------------------
;       Ganzzahldivision 2Byte / 1Byte
;-------------------------------------------------------------------------------

;Dividend: r1,r0 (LSB)
;Divisor:  r2
;Resultat: r0 (Quotient)
;Rest:     r1
;Achtung Ueberlauf wenn obere n Bit des Dividenden groesser gleich n Bit Divisor

.IF Flag_DIV21 == 1
DIV21:  push    r16
        in      r16,SREG
        push    r16
        
        ldi     r16,8           ;Schleifenzaehler init. (Bitzahl Quotient)
        
DIV21A: lsl     r0              ;Rotiere Dividend 1 Stelle nach links
        rol     r1              ;rechts wird Null reingeschoben
        brcs    DIV21B          ;Falls Carry muss Subtraktion erfolgen!
        cp      r1,r2           ;Subtraktion testen
        brlo    DIV21C          ;Ueberspringe Subtraktion, wenn kleiner

DIV21B: sub     r1,r2           ;Subtraktion durchfuehren
        inc     r0              ;Bit 0 im Dividenden setzen

DIV21C: dec     r16             ;Schleifenzaehler dekrementieren
        brne    DIV21A

        pop     r16
        out     SREG,r16
        pop     r16
        ret
.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahldivision 3Byte / 2Byte
;-------------------------------------------------------------------------------
;Dividend: r2,r1,r0 (LSB)
;Divisor:  r4,r3 (LSB)
;Resultat: r0 (Quotient)
;Rest:     r2:r1
;Achtung Ueberlauf wenn obere n Bit des Dividenden groesser gleich n Bit Divisor

.IF Flag_DIV32 == 1
DIV32:  push    r16
        in      r16,SREG
        push    r16

        ldi     r16,8           ;Schleifenzaehler SZ init. (Bitzahl Quotient)

DIV32A: lsl     r0              ;Rotiere Dividend 1 Stelle nach links
        rol     r1              ;rechts wird Null reingeschoben
        rol     r2
        brcs    DIV32B          ;Falls Carry muss Subtraktion erfolgen!
        cp      r1,r3           ;Subtraktion testen
        cpc     r2,r4
        brlo    DIV32C          ;Ueberspringe Subtraktion, wenn kleiner

DIV32B: sub     r1,r3           ;Subtraktion durchführen
        sbc     r2,r4
        inc     r0              ;Bit 0 im Dividenden setzen

DIV32C: dec     r16             ;Schleifenzaehler dekrementieren
        brne    DIV32A          ;Bleib in der Schleife bis SZ = 0

        pop     r16             ;Register wiederherstellen
        out     SREG,r16
        pop     r16
        ret                     ;Zurueck
.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahldivision 4Byte / 2Byte
;-------------------------------------------------------------------------------
;Dividend: r3,r2,r1,r0 (LSB)
;Divisor:  r5,r4 (LSB)
;Resultat: r1:r0 (LSB)
;Rest:     r3:r2 (LSB)
;Achtung Ueberlauf wenn obere n Bit des Dividenden groesser gleich n Bit Divisor

.IF Flag_DIV42 == 1
DIV42:  push    r16
        in      r16,SREG
        push    r16
        
        ldi     r16,16           ;Schleifenzaehler SZ init. (Bitzahl Quotient)
        
DIV42A: lsl     r0              ;Rotiere Dividend 1 Stelle nach links
        rol     r1              ;rechts wird Null reingeschoben
        rol     r2
        rol     r3
        brcs    DIV42B          ;Falls Carry muss Subtraktion erfolgen!
        cp      r2,r4           ;Subtraktion testen
        cpc     r3,r5
        brlo    DIV42C          ;Ueberspringe Subtraktion, wenn kleiner

DIV42B: sub     r2,r4           ;Subtraktion durchführen
        sbc     r3,r5
        inc     r0              ;Bit 0 im Dividenden setzen

DIV42C: dec     r16             ;Schleifenzaehler dekrementieren
        brne    DIV42A          ;Bleib in der Schleife bis SZ = 0

        pop     r16             ;Register wiederherstellen
        out     SREG,r16
        pop     r16
        ret                     ;Zurueck
.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahldivision 6Byte / 2Byte
;-------------------------------------------------------------------------------
;Dividend: r5,r4,r3,r2,r1,r0 (LSB)
;Divisor:  r7,r6 (LSB)
;Resultat: r3:r0 (LSB)
;Rest:     r5:r4 (LSB)
;Achtung Ueberlauf wenn obere n Bit des Dividenden groesser gleich n Bit Divisor

.IF Flag_DIV62 == 1
DIV62:  push    r16
        in      r16,SREG
        push    r16
        
        ldi     r16,32          ;Schleifenzaehler SZ init. (Bitzahl Quotient)
        
DIV62A: lsl     r0              ;Rotiere Dividend 1 Stelle nach links
        rol     r1              ;rechts wird Null reingeschoben
        rol     r2
        rol     r3
        rol     r4
        rol     r5
        brcs    DIV62B          ;Falls Carry muss Subtraktion erfolgen!
        cp      r4,r6           ;Subtraktion testen
        cpc     r5,r7
        
        brlo    DIV62C          ;Ueberspringe Subtraktion, wenn kleiner

DIV62B: sub     r4,r6           ;Subtraktion durchführen
        sbc     r5,r7
        inc     r0              ;Bit 0 im Dividenden setzen

DIV62C: dec     r16             ;Schleifenzaehler dekrementieren
        brne    DIV62A          ;Bleib in der Schleife bis SZ = 0

        pop     r16             ;Register wiederherstellen
        out     SREG,r16
        pop     r16
        ret                     ;Zurueck
.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahlmultiplikation 2Byte * 2Byte
;-------------------------------------------------------------------------------
;Multiplikationsbefehl des ATmega benoetigt r0 und r1 fuer Resultat
;Resultat der Multiplikation in r5,r4,r3,r2 (LSB)
;1. Zahl (a)in r7,r6 (LSB)
;2. Zahl (b) in  r9,r8 (LSB)

.IF Flag_MUL22 == 1
MUL22:  push    r16
        in      r16,SREG
        push    r16

        mul     r6,r8           ;Ergebnisregister mit 2 Produkten r6r8 und r7r9 
        movw    r2,r0           ;fuellen 
        mul     r7,r9 
        movw    r4,r0           
        mul     r6,r9           ;Produkt r6r7 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        mul     r7,r8           ;Produkt r6r7 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero

        pop     r16
        out     SREG,r16        ;Statusregister wiederherstellen
        pop     r16
        ret                     ;Zurueck
.ENDIF        

;-------------------------------------------------------------------------------
;       Ganzzahlmultiplikation 4Byte * 2Byte
;-------------------------------------------------------------------------------

;Multiplikationsbefehl des ATmega benoetigt r0 und r1 fuer Resultat
;Resultat der Multiplikation in r7,r6,r5,r4,r3,r2 (LSB)
;1. Zahl (a) in r11,r10,r9,r8 (LSB)
;2. Zahl (b) in  r13,r12 (LSB)

.IF Flag_MUL42 == 1
MUL42:  push    r16
        in      r16,SREG
        push    r16        
        
        mul     r8,r12          ;Ergebnisregister mit 2 Produkten r8r12 und 
        movw    r2,r0           ;r11r13 und Null fuellen
        mul     r11,r13 
        movw    r6,r0           ;movw    r7:r6,r1:r0     
        clr     r4
        clr     r5           
        
        mul     r8,r13          ;Produkt r8r13 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        mul     r9,r12          ;Produkt r9r12 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        
        mul     r10,r12         ;Produkt r10r12 versetzt zum Ergebnis addieren
        add     r4,r0
        adc     r5,r1
        adc     r6,Zero         ;Carry beruecksichtigen
        mul     r9,r13          ;Produkt r9r13 versetzt zum Ergebnis addieren
        add     r4,r0
        adc     r5,r1
        adc     r6,Zero         ;Carry beruecksichtigen
        
        mul     r11,r12         ;Produkt r11r12 versetzt zum Ergebnis addieren
        add     r5,r0
        adc     r6,r1
        adc     r7,Zero         ;Carry beruecksichtigen
        mul     r10,r13         ;Produkt r10r13 versetzt zum Ergebnis addieren
        add     r5,r0
        adc     r6,r1
        adc     r7,Zero         ;Carry beruecksichtigen
        
        pop     r16
        out     SREG,r16        ;Statusregister wiederherstellen
        pop     r16
        ret                     ;Zurueck
.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahlmultiplikation signed 
;       2Byte * 2Byte Achtung max. 32767*32767 (bzw. -32768)
;-------------------------------------------------------------------------------
;Multiplikationsbefehl des ATmega benoetigt r0 und r1 fuer Resultat
;Resultat der Multiplikation in r5,r4,r3,r2 (LSB)
;1. Zahl (a)in r7,r6 (LSB)
;2. Zahl (b) in  r9,r8 (LSB)

.IF Flag_MUL22S == 1
MUL22S: push    r16
        in      r16,SREG
        push    r16
        push    r17

        clt
        ldi     r17,0x80        ;Ueberpruefe Bit7 der 1. Zahl (negative Flag)
        and     r17,r7
        brne    MUL221
        rjmp    MUL222
MUL221: set                     ;T-Flag zeigt, dass Multiplikation negativ wird      
        com     r6              ;Zweierkomplement der 1. Zahl
        com     r7
        ldi     r16,1
        add     r6,r16
        adc     r7,Zero
        
MUL222: ldi     r17,0x80        ;Ueberpruefe Bit7 der 1. Zahl (negative Flag)
        and     r17,r9          ;Ueberpruefe Bit7 der 2. Zahl (negative Flag)
        brne    MUL223
        rjmp    MUL225          ;positiv dann weiter         

MUL223: com     r8              ;2 Zahl negativ: Zweierkomplement der 2. Zahl
        com     r9   
        ldi     r16,1
        add     r8,r16
        adc     r9,Zero
        brtc    MUL224          
        clt                     ;Falls T-Flag schon gesetzt, dann loeschen
        rjmp    MUL225           
MUL224: set

MUL225: mul     r6,r8           ;Ergebnisregister mit 2 Produkten r6r8 und r7r9 
        movw    r2,r0           ;fuellen 
        mul     r7,r9 
        movw    r4,r0           
        mul     r6,r9           ;Produkt r6r7 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        mul     r7,r8           ;Produkt r6r7 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero
        
        brtc    MUL226          
        com     r2              ;Resultat negativ! Zweierkomplement
        com     r3
        com     r4
        com     r5
        ldi     r16,1
        add     r2,r16
        adc     r3,Zero
        adc     r4,Zero
        adc     r5,Zero

MUL226: pop     r17
        pop     r16
        out     SREG,r16        ;Statusregister wiederherstellen
        pop     r16
        ret                     ;Zurueck
.ENDIF        

;-------------------------------------------------------------------------------
;       Ganzzahlmultiplikation  signed
;       4Byte * 2Byte Achtung max. 2147483647*32767 (bzw. -32768, -2147483648)
;-------------------------------------------------------------------------------

;Multiplikationsbefehl des ATmega benoetigt r0 und r1 fuer Resultat
;Resultat der Multiplikation in r7,r6,r5,r4,r3,r2 (LSB)
;1. Zahl (a) in r11,r10,r9,r8 (LSB)
;2. Zahl (b) in  r13,r12 (LSB)

.IF Flag_MUL42S == 1
MUL42S: push    r16
        in      r16,SREG
        push    r16        
        push    r17

        clt
        ldi     r17,0x80        ;Ueberpruefe Bit7 der 1. Zahl (negative Flag)
        and     r17,r11
        brne    MUL421
        rjmp    MUL422
MUL421: set                     ;T-Flag zeigt, dass Multiplikation negativ wird      
        com     r8              ;Zweierkomplement der 1. Zahl
        com     r9
        com     r10
        com     r11
        ldi     r16,1
        add     r8,r16
        adc     r9,Zero
        adc     r10,Zero
        adc     r11,Zero
        
MUL422: ldi     r17,0x80        ;Ueberpruefe Bit7 der 1. Zahl (negative Flag)
        and     r17,r13         ;Ueberpruefe Bit7 der 2. Zahl (negative Flag)
        brne    MUL423
        rjmp    MUL425          ;positiv dann weiter         

MUL423: com     r12             ;2 Zahl negativ: Zweierkomplement der 2. Zahl
        com     r13   
        ldi     r16,1
        add     r12,r16
        adc     r13,Zero
        brtc    MUL424          
        clt                     ;Falls T-Flag schon gesetzt, dann loeschen
        rjmp    MUL425           
MUL424: set

MUL425: mul     r8,r12          ;Ergebnisregister mit 2 Produkten r8r12 und 
        movw    r2,r0           ;r11r13 und Null fuellen
        mul     r11,r13 
        movw    r6,r0           ;movw    r7:r6,r1:r0     
        clr     r4
        clr     r5           
        
        mul     r8,r13          ;Produkt r8r13 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        mul     r9,r12          ;Produkt r9r12 versetzt zum Ergebnis addieren
        add     r3,r0
        adc     r4,r1
        adc     r5,Zero         ;Carry beruecksichtigen
        
        mul     r10,r12         ;Produkt r10r12 versetzt zum Ergebnis addieren
        add     r4,r0
        adc     r5,r1
        adc     r6,Zero         ;Carry beruecksichtigen
        mul     r9,r13          ;Produkt r9r13 versetzt zum Ergebnis addieren
        add     r4,r0
        adc     r5,r1
        adc     r6,Zero         ;Carry beruecksichtigen
        
        mul     r11,r12         ;Produkt r11r12 versetzt zum Ergebnis addieren
        add     r5,r0
        adc     r6,r1
        adc     r7,Zero         ;Carry beruecksichtigen
        mul     r10,r13         ;Produkt r10r13 versetzt zum Ergebnis addieren
        add     r5,r0
        adc     r6,r1
        adc     r7,Zero         ;Carry beruecksichtigen
        
        brtc    MUL426          
        com     r2              ;Resultat negativ! Zweierkomplement
        com     r3
        com     r4
        com     r5
        com     r6
        com     r7
        ldi     r16,1
        add     r2,r16
        adc     r3,Zero
        adc     r4,Zero
        adc     r5,Zero
        adc     r6,Zero
        adc     r7,Zero

MUL426: pop     r17        
        pop     r16
        out     SREG,r16        ;Statusregister wiederherstellen
        pop     r16
        ret                     ;Zurueck
.ENDIF

;-------------------------------------------------------------------------------
;	Hex nach ASCII (Hex in r24, ASCII in r25:r24)
;-------------------------------------------------------------------------------
.IF Flag_HEX2A2 == 1
HEX2A2:	push	r16
	push	r17
	ldi	r16,0x30
	ldi     r17,0x37
	mov	r25,r24
	andi	r25,0xF0
	swap	r25
	cpi	r25,10
	brlo    HEX2A21
	add     r25,r17
	rjmp    HEX2A22
HEX2A21: add    r25,r16
HEX2A22: andi	r24,0x0F
	cpi	r24,10
	brlo    HEX2A23
	add	r24,r17
	rjmp	HEX2A2R
HEX2A23: add	r24,r16
HEX2A2R: pop	r17
	pop	r16
	ret
.ENDIF	
;-------------------------------------------------------------------------------
;	BCD nach ASCII (BCD in r24, ASCII in r25:r24)
;-------------------------------------------------------------------------------
.IF Flag_BCD2A2 == 1
BCD2A2:
	push	r16
	push	r17	
	ldi	r16,0x30	
	mov	r17,r24
	andi	r24,0xF0
	swap	r24	
	add	r24,r16	
	mov	r25,r24
	mov	r24,r17
	andi	r24,0x0F
	add	r24,r16
	pop	r17
	pop	r16
	ret
.ENDIF	

;-------------------------------------------------------------------------------
;	Dezimal  bis 99 nach ASCII (Dezimal in r24, ASCII in r25:r24)
;-------------------------------------------------------------------------------
.IF Flag_DEC2A2 == 1
DEC2A2:	push	r16	
	mov	r0,r24
	clr	r1
	ldi	r16,10	
	mov	r2,r16	
	rcall	DIV21
	mov	r24,r1
	mov	r25,r0
	ldi	r16,0x30	
	add	r24,r16	
	add	r25,r16	
	pop	r16
	ret
.ENDIF	

;-------------------------------------------------------------------------------
;       Dezimal  bis 255 nach ASCII 
;       (Dezimal in r24, ASCII in SRAM-Tabelle TDEC2A + Nullbyte)
;-------------------------------------------------------------------------------
.IF Flag_DEC2A3 == 1
DEC2A3: push    r16     
        push    r17
        push    r18
        push    r24

        ldi     r18,0x30
        mov     r0,r24
        mov     r1,Zero        
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21
        add     r0,r18
        sts     TDEC2A+0,r0
        mov     r0,r1
        clr     r1
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        add     r0,r18
        sts     TDEC2A+1,r0
        add     r1,r18
        sts     TDEC2A+2,r1
        sts     TDEC2A+3,Zero

        pop     r24
        pop     r18
        pop     r17
        pop     r16
        ret
.ENDIF  

;-------------------------------------------------------------------------------
;	Dezimal  bis 65535 nach ASCII 
;	(Dezimal in r25:r24, ASCII in SRAM-Tabelle TDEC2A + Nullbyte)
;-------------------------------------------------------------------------------
.IF Flag_DEC2A5 == 1
DEC2A5:	push	r16	
	push	r17
	push	r18
        push    r24
        push    r25
	ldi	r18,0x30
	mov	r0,r24
	mov	r1,r25
	clr	r2
	ldi	r16,LOW(10000)
	ldi	r17,HIGH(10000)
	mov	r3,r16
	mov	r4,r17
	rcall	DIV32
	add	r0,r18
	sts	TDEC2A+0,r0
	mov	r0,r1
	mov	r1,r2
	clr	r2
	ldi	r16,LOW(1000)
	ldi	r17,HIGH(1000)
	mov	r3,r16
	mov	r4,r17
	rcall	DIV32
	add	r0,r18
	sts	TDEC2A+1,r0
	mov	r0,r1
	mov	r1,r2
	ldi	r16,100
	mov	r2,r16
	rcall	DIV21
	add	r0,r18
	sts	TDEC2A+2,r0
	mov	r0,r1
	clr	r1
	ldi	r16,10
	mov	r2,r16
	rcall	DIV21
	add	r0,r18
	sts	TDEC2A+3,r0
	add	r1,r18
	sts	TDEC2A+4,r1
	sts     TDEC2A+5,Zero
        pop     r25
        pop     r24
	pop	r18
	pop	r17
	pop	r16
	ret
.ENDIF	

;-------------------------------------------------------------------------------
;	ASCII (BCD,2 Byte) nach Dezimal/Binaer 
;       (ASCII-Stellen nur 0-9; ASCII in r25:r24, Dezimal in r24)
;-------------------------------------------------------------------------------
.IF Flag_ASCB2D2 == 1
ASCB2D2: push	r16	
	ldi	r16,10	
	subi	r25,0x30	
	subi	r24,0x30	
	mul	r25,r16
	add	r24,r0
	pop	r16
	ret
.ENDIF	

;-------------------------------------------------------------------------------
;       ASCII (HEX,2 Byte) nach Dezimal/Binaer
;       (ASCII-Stellen 0-F; ASCII in r25:r24, Dezimal in r24)
;-------------------------------------------------------------------------------
.IF Flag_ASCH2D2 == 1
ASCH2D2: subi  r25,0x30        
        subi    r24,0x30        
        cpi     r25,0x0A
        brlo    ASCH2D22
        cpi     r25,0x31
        brlo    ASCH2D21        ;Grossbuchstabe A-F
        subi    r25,0x27        ;Kleinbuchstabe a-f
        rjmp    ASCH2D22
ASCH2D21: subi r25,0x07        
ASCH2D22: cpi  r24,0x0A
        brlo    ASCH2D24
        cpi     r24,0x31
        brlo    ASCH2D23        ;Grossbuchstabe A-F
        subi    r24,0x27        ;Kleinbuchstabe a-f
        rjmp    ASCH2D22
ASCH2D23: subi r24,0x07        
ASCH2D24: lsl  r25              ;Highbyte*16
        lsl     r25
        lsl     r25
        lsl     r25
        add     r24,r25
        ret
.ENDIF  

;-------------------------------------------------------------------------------
;	ASCII (BCD,4 Byte) nach Dezimal/Binaer 
;       (ASCII in Tabelle Tabellenzeiger X Dezimal in r25:r24)
;	Kann bei durchgehenden Tabellen mehrfach hintereinander verwendet werden
;	ohne Neuinitialisierung des Tabellenzeigers!
;-------------------------------------------------------------------------------
.IF Flag_ASCB2D4 == 1
ASCB2D4: push	r16
	push	r17

	ld 	r16,X+		;oberste Stelle mit 1000 multiplizieren
	subi 	r16,0x30
	ldi 	r17,10
	mul	r16,r17	
	mov	r16,r0
	ldi 	r17,100
	mul	r16,r17
	mov	r24,r0
	mov	r25,r1
	ld 	r16,X+		;*100
	subi 	r16,0x30
	ldi 	r17,100
	mul	r16,r17		
	add	r24,r0
	adc	r25,r1
	ld 	r16,X+		;*10
	subi 	r16,0x30
	ldi 	r17,10
	mul	r16,r17		
	add	r24,r0
	adc	r25,r1
	ld 	r16,X+		;addieren	
	subi 	r16,0x30
	add	r24,r16
	adc	r25,Zero

	pop	r17
	pop	r16
	ret
.ENDIF	

